

mydata <- read.csv("ADVANCED_DM", head = T, stringsAsFactors=FALSE)

outPath <- "FJ_OUTS_FOLDER"


packages<-function(x){
  x<-as.character(match.call()[[2]])
  if(!require(x,character.only=TRUE)){
    install.packages(pkgs=x,repos="http://cran.r-project.org")
    require(x,character.only=TRUE)
  }
}

packages(readr)
packages(pheatmap)
packages(DT)
packages(ggplot2)
packages(shinyWidgets)
packages(ggsignif)
packages(broom)
packages(ggpubr)
packages(gginnards)
packages(gtools)
packages(plotly)
packages(RColorBrewer)

packages(visNetwork)
packages(igraph)
packages(shiny)
packages(shinydashboard)
packages(dplyr)
packages(magrittr)
packages(data.table)
packages(htmlwidgets)
packages(viridisLite)
packages(colourvalues)
packages(tidyverse)
packages(png)
packages(widgetframe)
packages(stats)

packages(pheatmap)
packages(ggplot2)
packages(cowplot)

###############################
##      Make Test Plots    ##
###############################
# 
#   fData <- mydata
# 
#   j <- ncol(fData)
# 
#   h <- nrow(fData)
#   selectedParameterNumber <- j-4
# 
# 
## need to move the titles for pops into the "ids"...
# for (l in 1: nrow(mydata)){
#   niceName <- as.character(mydata$NAME[[l]])
#   replaceMe <- as.character(mydata$ID[[l]])
#   
#   for (k in 1:nrow(mydata)){
#     if(mydata$ID[[k]] == replaceMe){
#       mydata$ID[[k]] <- niceName
#     }
#     if(mydata$PARENT_ID[[k]] == replaceMe){
#       mydata$PARENT_ID[[k]] <- niceName
#     }
#   }
# }



###############################
##     Settings    ##
###############################
n <- ncol(mydata)
if(n >= 5){

    if(n >= 5){
        colers <- colnames(mydata)
        for(m in 1:length(colers)){
            colers[m] <- gsub("^X\\.","", colers[m])
        }
        colnames(mydata) <- colers
        Parameters <- colnames(mydata[,5:n])
    } else {
        Parameters = NULL;
    }

    Parameters <- colnames(mydata[,5:n])


} else {
    Parameters = NULL;
}
Parameters <- c("NONE",Parameters)

Clusters <- mydata$NAME

####################
# Set Menu Options #
####################
vbPalette <- c("Dark2","Accent","Pastel2","Set2", "Pastel1", "Set1","Paired","Set3","Spectral","BrBG","PiYG","PRGn","PuOr")
sbPalette <- c("magma", "plasma", "inferno", "viridis", "cividis") ##, "rainbow")
# sbPalette <- c("Dark2","Accent","Pastel2","Set2", "Pastel1", "Set1","Paired","Set3","Spectral","BrBG","PiYG","PRGn","PuOr")

clusterType <- c("ward", "single", "complete", "average", "mcquitty", "median", "centroid")

clusterDistances <- c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski") ##"correlation", 


####################
#    Functions     #
####################


cluster_mat = function(mat, distance, method){
  if(!(method %in% c("ward", "single", "complete", "average", "mcquitty", "median", "centroid"))){
    stop("clustering method has to one form the list: 'ward', 'single', 'complete', 'average', 'mcquitty', 'median' or 'centroid'.")
  }
  if(!(distance %in% c("euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski")) & class(distance) != "dist"){    ## "correlation", 
    print(!(distance %in% c("correlation", "euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski")) | class(distance) != "dist")
    stop("distance has to be a dissimilarity structure as produced by dist or one measure  form the list: 'correlation', 'euclidean', 'maximum', 'manhattan', 'canberra', 'binary', 'minkowski'")
  }
  # if(distance == "correlation"){
  #   d = as.dist(1 - cor(t(mat)))
  # }
  else{
    if(class(distance) == "dist"){
      d = distance
    }
    else{
      d = dist(mat, method = distance)
    }
  }
  
  return(stats::hclust(d, method = method))
}


#######################
##   Shiny Part:     ##
#######################
CorrelationNetwork <- function() {
  require(shiny)
  # shinyApp(ui = ui, server = server)
  library(shiny)
  library(visNetwork)
  library(shinyWidgets)
  library(DT)
  library(magrittr)
  library(data.table)
  library(shinydashboard)
  library(stats)
  library(tidyverse)
  library(data.table)
  library(magrittr)
  
  library(dplyr)
  library(pheatmap)
  library(viridisLite)
  library(ggplot2)
  library(cowplot)
  library(RColorBrewer)
  library(ggsignif)
  library(broom)
  library(ggpubr)
  library(gginnards)
  library(gtools)
  
  # library(plotly)
  # options(shiny.trace=TRUE)
  
  shinyApp(
    
    ###################
    ##   Build UI    ##
    ###################
    ui <- dashboardPage(
      dashboardHeader(title = "Population Network"),
      
      ########################
      ##   Build SideBar    ##
      ########################
      mysidebar <- dashboardSidebar(
        width = 190,
        sidebarMenu(
          id = 'tabs',
          
          actionButton("refreshPlot",
                       label = "RECALCULATE"),
          
          menuItem(text = "Data",
                   tabName = "parentData",
                   icon = icon("table"),
                   menuSubItem(text = "", tabName = "data", icon = NULL
                               # selected = T
                   ),
                   
                   selectInput("parameters",
                               label = "Select Parameters to Plot",
                               choices = as.character(Parameters),
                               multiple = T,
                               selectize = F,
                               selected = as.character("NONE")),
                   
                   selectInput("clusters",
                               label = "Choose Clusters to Plot",
                               choices = Clusters,
                               selected = Clusters,
                               multiple = T,
                               selectize = F),
                   
                   selectInput("size",
                               label = "Node Size Relative To:",
                               choices = c("NONE", "Count"),
                               selected = "Count",
                               multiple = F,
                               selectize = T)
          ),
          
          menuItem(text = "MiST",
                   tabName = "MiST",
                   startExpanded = F,
                   icon = icon("cloudsmith"), 
                   
                   menuSubItem("", tabName ="network", icon = NULL),
                   
                   checkboxInput("norm",
                                 label = "Normalize Radar Plot Expression",
                                 T),
                   
                   selectInput("plotColor",
                               label = "Select Univariate Colors",
                               choices = sbPalette,
                               multiple = FALSE,
                               selectize = TRUE),
                   
                   selectInput("vbPalette",
                               label = "Select Radar Color Palette",
                               choices = vbPalette,
                               multiple = FALSE,
                               selectize = TRUE)
          ),
          
          menuItem(text = "Options",
                   tabName = "hireTab",
                   startExpanded = F,
                   icon = icon("cloudsmith"), ## TODO - Give this a better icon

                   # checkboxInput("hire",
                   #               label = "Force Hierarchy",
                   #               T),
                   # selectInput("clusterMethod",
                   #             label = "Select Clustering Method",
                   #             choices = clusterType,
                   #             multiple = FALSE,
                   #             selectize = TRUE),
                   # selectInput("clusterDistance",
                   #             label = "Select Clustering Criteria",
                   #             choices = clusterDistances,
                   #             multiple = FALSE,
                   #             selectize = TRUE
                   # ),
                   checkboxInput("freeze",
                               label = "Freeze Nodes",
                               F
                   ),
                   checkboxInput("circular",
                                 label = "Circular",
                                 F
                   ),
                   checkboxInput("autoOrg",
                                 label = "Order Nodes",
                                 T
                   )
          ),
          
          # selectInput("textOrientation",
          #             label = "Text Orientation",
          #             choices = orientationOptions,
          #             multiple = FALSE,
          #             selected = "auto"),
          # 
          # selectInput("normy",
          #             label = "Slice Normalization",
          #             choices = sliceNorm,
          #             multiple = F,
          #             selected = "remainder"),
          # 
          # numericInput("maxDepth",
          #              label = "Maximum Layers",
          #              value = NULL,
          #              max = 20,
          #              min = 2,
          #              step = 1
          # ),
          
          downloadButton("print", label = "PRINT HTML"),
          actionButton("show", "Help")
          # downloadButton("print", label = "PRINT HTML")
        )
      ),
      
      ######################
      ## Build Dashboard  ##
      ######################
      dashboardBody(
        
        ## These CSS sytles will giv us the full viewport height for graphs specified:
        tags$head(tags$style("#network{height:90vh !important;}")),
        # tags$head(tags$style("#violins{height:90vh !important;}")),
        
        tabItems(
          tabItem(tabName = "data",
                  DTOutput(outputId = "table1")
          ),
          
          tabItem(tabName = "network",
                  visNetworkOutput("network")
          )
          # tabItem(tabName = "print",
          # plotlyOutput("print")
        )
      ),
      
      #####################
      ##   Build Tags    ##
      #####################
      tags$head(
        tags$link(rel="icon", href="http://fjinstallers.s3-us-west-2.amazonaws.com/Plugins/MiST_favicon.png"),
        tags$style(HTML('

        /* logo */
        .skin-blue .main-header .logo {
                              background-color: #696969;
        }

        #                       /* logo when hovered */
        # .skin-blue .main-header .logo:hover {
        #                       background-color: ##696969;
        #                       }

        /* navbar (rest of the header) */
        .skin-blue .main-header .navbar {
                              background-color: #696969;
                              }

        /* main sidebar */
        .skin-blue .main-sidebar {
                              background-color: #696969;
                              }

        /* active selected tab in the sidebarmenu */
        .skin-blue .main-sidebar .sidebar .sidebar-menu .active a{
                              background-color: #b6a4b6;
                              }

        /* other links in the sidebarmenu */
        .skin-blue .main-sidebar .sidebar .sidebar-menu a{
                              background-color: #b0b0b0;
                              color: #000000;
                              }

        /* other links in the sidebarmenu when hovered */
         .skin-blue .main-sidebar .sidebar .sidebar-menu a:hover{
                              background-color: #ff69b4;
                              }
        /* toggle button when hovered  */
         .skin-blue .main-header .navbar .sidebar-toggle:hover{
                              background-color: #ff69b4;
                              }
        /* body */
          .content-wrapper, .right-side {
                              background-color: #ffffff;
                              }
      ')))
      
    ),
    
    
    
    #################
    # Make a Server #
    #################
    
    server <- function(input, output, session) {
      
      session_store <- reactiveValues()
      
      observeEvent(input$sidebarItemExpanded, {
        test <- input$sidebarItemExpanded
        if(test == "MiST"){
          updateTabItems(session,"tabs","network")
        } else if (test == "parentData") {
          updateTabItems(session,"tabs","table1")
        }
      })
      
      #############################
      ##      Make Table Data    ##
      #############################
      filterDataTable <- reactive({
          n <- ncol(mydata)
          if(n >= 5){
              param <- colnames(mydata[,5:n])
          } else {
              param = NULL;
          }
        
        clust <- mydata$NAME
        
        PARAM_SELECTION <- input$parameters
        CLUSTER_SELECTION <- input$clusters
        groupedInput <- mydata %>%
          filter(clust %in% CLUSTER_SELECTION) ## Cleaning up your rows

          n <- ncol(mydata)
          if(n >= 5){
              # param <- colnames(mydata[,5:n])
              groupedInput[,5:n] <- round(groupedInput[,5:n],2)
          }
        
        if(input$parameters == "NONE" && input$size == "NONE"){ 
          parametersToKeep <- c("NAME")
        } else if(input$parameters != "NONE" && input$size == "NONE") {
          parametersToKeep <- c("NAME", PARAM_SELECTION)
        } else if(input$parameters == "NONE" && input$size != "NONE"){
          parametersToKeep <- c("NAME", "COUNT")
        } else {
          parametersToKeep <- c("NAME", "COUNT", PARAM_SELECTION)
        }

          if(n >= 5){
              GI <- select(groupedInput, all_of(parametersToKeep)) # Cleaning up your columns
          } else {
              GI <- groupedInput
          }

        return(GI)
      })
      
      #################################
      ##      Make Main DataFrame    ##
      #################################
      # This will return the df
      filterData <- reactive({

          n <- ncol(mydata)
          if(n >= 5){
              param <- colnames(mydata[,5:n])
          } else {
              param = NULL;
          }
          # param <- colnames(mydata[,5:n])
          # Parameters <- c("NONE",Parameters)
        
        clust <- mydata$NAME
        
        PARAM_SELECTION <- input$parameters
        CLUSTER_SELECTION <- input$clusters
        groupedInput <- mydata %>%
          filter(clust %in% CLUSTER_SELECTION) ## Cleaning up your rows

          n <- ncol(mydata)
          if(n >= 5){
              # param <- colnames(mydata[,5:n])
              groupedInput[,5:n] <- round(groupedInput[,5:n],2)
          }
        
        if(input$parameters == "NONE"){ 
          parametersToKeep <- c("ID","PARENT_ID","NAME", "COUNT")
        } else {
          parametersToKeep <- c("ID","PARENT_ID","NAME", "COUNT", PARAM_SELECTION)
        }

          if(n >= 5){
              GI <- select(groupedInput, all_of(parametersToKeep)) # Cleaning up your columns
          } else {
              GI <- groupedInput
          }

        return(GI)
      })
      
      observeEvent(input$show, {
        showNotification("To Save as HTML First Download Pandoc Application Here: ",
                         a(href="https://pandoc.org/installing.html", "pandoc.org")) ## target="_blank">pancod.org</a>)
      })
      
      
      #################################
      ##  Make MultiColor Palette    ##
      #################################
      colorz <- eventReactive(input$refreshPlot,{
        getPalette <- colorRampPalette(brewer.pal(8, input$vbPalette))(length(input$parameters))
        return(getPalette)
      })
      
      
      
      ###############################
      ##      Make Star Plots    ##
      ###############################
      
      starz <- eventReactive(input$refreshPlot,{  ## Only invoke this option if two or more parameters are selected...
        
        unlink(paste0(outPath,"/www"), recursive = TRUE)
        dir.create(paste0(outPath, "/www"))
        
        fData <- filterData()
        
        j <- ncol(fData)
        h <- nrow(fData)
        selectedParameterNumber <- j-4
        
        
        if(input$norm){
          for (q in 5:j){
            fData[,q] <- (abs(fData[,q])/max(fData[,q]))*100
          }
        }
        
        ## remove plots sitting at: paste0(outPath, "/www")
        # unlink( paste0(outPath,"/www/plot_",1:selectedParameterNumber,".png") )

          getPalette <- colorz()
        
        PlotIndex <- list()
        MyPlots <- list()  # new empty list

        # Create dataset - iterate over the number of parameters here:
        for (s in 1:h){
          # s=9
          data <- data.frame(
            id=colnames( fData[s,5:j] ),
            individual=colnames( fData[s,5:j] ),
            value=c( t(fData[s,5:j]) )
          )

          # Make the plot
          p <- ggplot(data, aes(x=as.factor(id), y=value)) +       # Note that id is a factor. If x is numeric, there is some space between the first bar
            geom_bar(stat="identity",alpha = 0.8, fill = getPalette) + ## , inherit.aes = F, aes(x=value, y=value) ) +   # This add the bars with a color
            ylim(-0.05*max(fData[,5:j]),max(fData[,5:j])) +   # Limits of the plot = very important. The negative value controls the size of the inner circle, the positive one is useful to add size over each bar
            theme_minimal() +     # Custom the theme: no axis title and no cartesian grid
            theme(
              legend.position = "none",
              axis.text = element_blank(),
              axis.title = element_blank(),
              panel.grid = element_blank(),
              plot.margin = unit(rep(-2,4), "cm")     # This remove unnecessary margin around plot
            ) +
            coord_polar(start = 0)   # This makes the coordinate polar instead of cartesian.
          # p
          
          PlotIndex[[s]] <- fData[s,1]
          ggsave(paste0(outPath, "/www/plot_",s,".png"),p, width = 128, height = 128, units = "mm")
          MyPlots[[s]] <- paste0("File://",outPath,"/www/plot_",s,".png")
        }
        binder <- as.data.frame(rbind(PlotIndex,MyPlots))
        return(binder)
      })
      
      
      ###############################
      ##      Make Network Plot    ##
      ###############################
      networkPlot <- eventReactive(input$refreshPlot, {
        
        #####################
        ## Data Management ##
        #####################
        DATA <- filterData()
        
        j <- ncol(DATA)
        h <- nrow(DATA)
        selectedParameterNumber <- j-4
        
        path_to_images <- paste0("/plot_") ##
        
        twoParam <- F
        if(length(input$parameters) > 1){
          twoParam <- T
        }
        
        graph <- graph.data.frame(DATA, directed=input$autoOrg)
        networks <- clusters(as.undirected(graph))
        V(graph)$network <- networks$membership
        
        
        ##########################
        ##  ReDefine Hierarchy  ##
        ##########################
        
        # HIRE <- F
        # if (input$hire){
        #   HIRE <- T
        # }
        
        # if(HIRE){
        #   
        #   
        #   # Do clustering
        #   # if(cluster_rows){
        #   tree_row <- cluster_mat( DATA[,5:j], distance = eval(input$clusterDistance), method = eval(input$clusterMethod) )
        #   mat <- DATA[tree_row$order, , drop = FALSE]
        #   # }
        #   # else{
        #   #   tree_row = NA
        #   #   treeheight_row = 0
        #   # }
        #   
        #   # if(cluster_cols){
        #   #   tree_col = cluster_mat(t(mat), distance = clustering_distance_cols, method = clustering_method)
        #   #   mat = mat[, tree_col$order, drop = FALSE]
        #   # }
        #   # else{
        #   #   tree_col = NA
        #   #   treeheight_col = 0
        #   # }
        # }
        
        
        
        ################
        ## Set Names  ##
        ################
        
        DATA_NAME = data.frame(name = character(),
                               stringsAsFactors = F )
        for (e in 1:length(networks$membership)){   ## Assign a name from the ID
          E_SET <- F
          for(f in 1:length(DATA$ID)){
            if (names(networks$membership[e]) == DATA$ID[f]){
              DATA_NAME[e,1] <- DATA$NAME[f]
              E_SET <- T
            }
          }
          if (!E_SET){
            DATA_NAME[e,1] <- names(networks$membership[e])
          }  
        }
        
        
        ##########################
        ## Set Names - part ii  ##
        ##########################
        for (f in 1:nrow(DATA_NAME)){  ## Assign a unique name from the ID
          counter <- 0
          for (g in 1:nrow(DATA_NAME)){
            if ( (DATA_NAME[g,1] == DATA_NAME[f,1]) && (g != f) ){
              DATA_NAME[g,1] <- paste0(DATA_NAME[g,1],".0",counter)
              counter <- counter + 1
            }
          }
        }
        
        
        ################
        ## Set Values ##
        ################
        if((input$parameters != "NONE") && (!twoParam)){ ## Assign a value from the selected parameter
          
          VALUE_SELECTED = data.frame(value = character(),
                                      stringsAsFactors = F )
          for (e in 1:length(networks$membership)){
            E_SET <- F
            for(f in 1:length(DATA$ID)){
              if (names(networks$membership[e]) == DATA$ID[f]){
                VALUE_SELECTED[e,1] <- DATA[f,5]
                E_SET <- T
              }
            }
            if (!E_SET){
              VALUE_SELECTED[e,1] <- 0
            }  
          }
          
          V(graph)$color <- VALUE_SELECTED[,1]
          V(graph)$value <- VALUE_SELECTED[,1]
          
        }
        
        
        ################
        ## Set Images ##
        ################
        if((input$parameters != "NONE") && (twoParam)){ ## Assign an image from the selected parameters and IDs
          
          STARS <- starz()
          
          IMAGES <- data.frame(
            plotLocation = as.character(),
            stringsAsFactors = F
          )
          
          for (e in 1:length(networks$membership)){
            E_SET <- F
            for(f in 1:length(STARS)){
              if (names(networks$membership[e]) == STARS[1,f]){
                IMAGES[e,1] <- STARS[2,f]
                E_SET <- T
              }
            }
            if (!E_SET){
              IMAGES[e,1] <- paste0("/brokenImageImage.png")
            }  
          }
        }
        
        ################
        ## Set Counts ##
        ################
        if(input$size != "NONE"){   # Set counts for each member of the list
          MaxSize <- max( DATA[,4] )
          COUNT = data.frame(value = character(),
                             stringsAsFactors = F )
          for (e in 1:length(networks$membership)){
            E_SET <- F
            for(f in 1:length(DATA$ID)){
              if (names(networks$membership[e]) == DATA$ID[f]){
                COUNT[e,1] <- DATA[f,4]
                E_SET <- T
              }
            }
            if (!E_SET){
              COUNT[e,1] <- MaxSize
            }  
          }
          V(graph)$size <- COUNT[,1]
        }
        V(graph)$id <- DATA_NAME[,1]
        V(graph)$name <- DATA_NAME[,1]
        
        B <- input$freeze
        
        C <- input$circular
        
        ########################
        ## Create the Network ##
        ########################
        if( (input$parameters != "NONE") && (!twoParam) ){    
          ############################
          ## one parameter selected ##
          ############################
          
          df <- data.frame(a = length(V(graph)$value), x = as.numeric(V(graph)$value))
          df$col <- colourvalues::colour_values(df$x, palette = input$plotColor)
          
          bar_plot <- function(df) {
            barplot( height = df[["a"]], col = df[["col"]], border = NA, space = 0, yaxt = 'n')
          }
          
          pal <- eval(parse(text = input$plotColor))
          # pal <- eval(parse(text = "plasma"))
          V(graph)$color <- df$col
          
          if(input$size != "NONE") {
            nodes <- data.frame(id = V(graph)$id, 
                                title = V(graph)$name, 
                                group = V(graph)$network, 
                                color = V(graph)$color,
                                value = V(graph)$size)
          } else {
            nodes <- data.frame(id = V(graph)$id, 
                                title = V(graph)$name, 
                                group = V(graph)$network, 
                                color = V(graph)$color)
          }
          
          nodes <- nodes[order(nodes$id, decreasing = F),]
          edges <- get.data.frame(graph, what="edges")
          
          # test <<- V(graph)
          
          dff <- df[order(df$x),c("a","x","col")]
          
          ledges <- data.frame(color = rev(dff$col))
          
          visNetwork(nodes, edges) %>%
            visOptions(highlightNearest = TRUE, selectedBy = NULL) %>%
            visNodes(color = "black") %>%
            visEdges(color="black", smooth = F) %>%
            visInteraction(navigationButtons = TRUE)  %>%
            visPhysics(stabilization = FALSE) %>%
            { if( B && !C ) visIgraphLayout(.) else if ( C ) visIgraphLayout( . , layout = "layout_in_circle") else . } %>%
            visLegend(main = input$parameters, useGroups = F, addNodes = ledges, zoom = F)
          
          
          #####
          ###  Confirm the PNG files are correct per population ##
          ##### 
        } else if( (input$parameters != "NONE") && (twoParam) ){    
          ######################################
          ## More than one parameter selected ##
          ######################################
          df <- data.frame(a = ncol(DATA[,5:j]), x = colnames(DATA[,5:j]))
          df$col <- colorz()
          ledges <- data.frame(parameter = input$parameters,
                               color = df$col,
                               label = colnames(DATA[,5:j]))
          
          nodes <- NULL
          if(input$size != "NONE") {
            nodes <- data.frame(id = V(graph)$id, 
                                title = V(graph)$name, 
                                group = V(graph)$network, 
                                color = list(background = "black", border = "white"),
                                value = V(graph)$size,
                                shape = "circularImage",
                                image = paste0(path_to_images, 1:nrow(IMAGES), ".png?t=", Sys.time()),
                                brokenImage = "https://encrypted-tbn0.gstatic.com/images?q=tbn%3AANd9GcSFYWyxKQHv-RdYu2VRYbAaQmo-_b-FiaerT-FKj3nmTFPExoge&usqp=CAU"
            )
          } else {
            nodes <- data.frame(id = V(graph)$id, 
                                title = V(graph)$name, 
                                group = V(graph)$network,
                                color = list(background = "black", border = "white"),
                                shape = "circularImage",
                                image = paste0(path_to_images, 1:nrow(IMAGES), ".png?t=", Sys.time()),
                                brokenImage = "https://encrypted-tbn0.gstatic.com/images?q=tbn%3AANd9GcSFYWyxKQHv-RdYu2VRYbAaQmo-_b-FiaerT-FKj3nmTFPExoge&usqp=CAU")
          }
          
          nodes <- nodes[order(nodes$id, decreasing = F),]
          edges <- get.data.frame(graph, what="edges")
          
          visNetwork(nodes, edges) %>%
            visOptions(highlightNearest = TRUE, selectedBy = NULL) %>%
            visNodes(color = list(border = "white"),shapeProperties = list(useBorderWithImage = FALSE)) %>%
            visEdges(color="black", smooth = F) %>%
            visInteraction(navigationButtons = TRUE)  %>%
            visPhysics(stabilization = FALSE)  %>%
            { if( B && !C ) visIgraphLayout(.) else if ( C ) visIgraphLayout( . , layout = "layout_in_circle") else . } %>%
            visLegend(ncol = 1, 
                      main = "Radar Parameters", 
                      useGroups = F, 
                      addNodes = ledges, 
                      zoom = F)  ## TODO - Need to create a legend for the parameters in the Star Plots...
          
        } else {   
          #############################
          ## No parameters selected  ##
          #############################
          if(input$size != "NONE") {
            nodes <- data.frame(id = V(graph)$id, 
                                title = V(graph)$name, 
                                group = V(graph)$network, 
                                value = V(graph)$size)
          } else {
            nodes <- data.frame(id = V(graph)$id, 
                                title = V(graph)$name, 
                                group = V(graph)$network)
          }
          
          nodes <- nodes[order(nodes$id, decreasing = F),]
          edges <- get.data.frame(graph, what="edges")
          
          visNetwork(nodes, edges) %>%
            visOptions(highlightNearest = TRUE, selectedBy = NULL) %>%
            visNodes(color = "black") %>%
            visEdges(color="black", smooth = F) %>%
            visInteraction(navigationButtons = TRUE)  %>%
            visPhysics(stabilization = FALSE)  %>%
            { if( B && !C ) visIgraphLayout(.) else if ( C ) visIgraphLayout( . , layout = "layout_in_circle") else . } %>%
            visGroups(groupname = "1", color = "black")
        }
      })
      
      ###############
      ##  OUTPUTS  ##
      ###############
      output$network <- renderVisNetwork({
        session_store$plt <- networkPlot()
        session_store$plt
      })
      
      output$print <- downloadHandler(
        filename = function() {
          paste("Network.html", sep = "")
          # paste("File.html", sep = "")
        },
        content = function(file) {
          saveWidget(as_widget(session_store$plt), file, selfcontained = T)
        }
      )
      
      output$table1 <- DT::renderDataTable(datatable(filterDataTable(),
                                                 extensions = 'Buttons',
                                                 plugins = 'natural',
                                                 options = list(
                                                   pageLength = -1, info = FALSE,
                                                   lengthMenu = list(c(15, -1), c("15", "All")),
                                                   columnDefs = list(
                                                     list(type = 'natural',
                                                          targets = 'all')
                                                   ),
                                                   dom = 'Bfrtip',
                                                   buttons = list(
                                                     list(
                                                       extend = "copy",
                                                       text = "COPY",
                                                       title = "MiST FlowJoExchange Plugin - Table Export"
                                                     ))
                                                 ))
      )
      
      output$print <- downloadHandler(
        filename = function() {
          paste('network-', Sys.Date(), '.html', sep='')
        },
        content = function(con) {
          networkPlot() %>%
            visSave(con, selfcontained = T, background = "white")
        }
      )
      
      ###################
      ## Final Options ##
      ###################
      launch.browser=TRUE
      session$onSessionEnded(stopApp)
    })
}
CorrelationNetwork()

